require 'rails_helper'

RSpec.describe User do
  describe 'create_user_from_hash' do
    before do
      rub_default_admin_set_title = ENV.fetch('RUB_ADMIN_SET_TITLE', 'RUB publication workflow')
      crc_default_admin_set_title = ENV.fetch('CRC_ADMIN_SET_TITLE', 'CRC 1280 Publishing Workflow')
      admin_set_1 = AdminSet.where(title: [rub_default_admin_set_title]).first
      admin_set_1 = FactoryBot.create(:admin_set, title: [rub_default_admin_set_title]) unless admin_set_1.present?
      admin_set_2 = AdminSet.where(title: [crc_default_admin_set_title]).first
      admin_set_2 = FactoryBot.create(:admin_set, title: [crc_default_admin_set_title]) unless admin_set_2.present?
    end
    context "create" do
      after do
        @user.delete if @user.present?
      end
      it 'creates a new user with all the properties' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "password",
          name: "Example User",
          saml_id: "hyrax:1234",
          orcid: "0000-0000-0000-0001",
          role: "crc_1280_group_manager",
          group_id: "vd66vz88c"
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:email]}")
        expect(messages).to include("Assigned user #{user_attributes[:email]} role crc_1280_vd66vz88c_manager")
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.display_name).to eq user_attributes[:name]
        expect(@user.provider).to eq 'saml'
        expect(@user.uid).to eq user_attributes[:saml_id]
        expect(@user.orcid).to eq "https://orcid.org/#{user_attributes[:orcid]}"
      end

      it 'creates a new user with email and password' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "password",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:email]}")
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end

      it 'creates a new user with just email' do
        user_attributes = {
          email: "example_user@hyrax",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:email]}")
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end

      it 'creates a new user with just saml id' do
        user_attributes = {
          saml_id: "hyrax:1234",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:saml_id]}@saml")
        expect(@user.email).to eq "#{user_attributes[:saml_id]}@saml"
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to eq 'saml'
        expect(@user.uid).to eq user_attributes[:saml_id]
        expect(@user.orcid).to be nil
      end

      it 'creates a new user with just orcid' do
        user_attributes = {
          orcid: "0000-0000-0000-0001",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:orcid]}@orcid")
        expect(@user.email).to eq "#{user_attributes[:orcid]}@orcid"
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to eq "https://orcid.org/#{user_attributes[:orcid]}"
      end

      it "does not create a user with no email, saml id or orcid" do
        user_attributes = {
          name: "Example user",
          password: "password",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be false
        expect(messages).to include("Error: Not creating user. User has no email, saml id, or orcid.")
        expect(@user).to be nil
      end

      it 'creates a new user with corresponding role and workflow role' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "password",
          name: "Example User",
          role: "crc_1280_group_manager",
          group_id: "vd66vz88c"
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be true
        expect(messages).to include("Creating user #{user_attributes[:email]}")
        expect(messages).to include("Assigned user #{user_attributes[:email]} role crc_1280_vd66vz88c_manager")
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.display_name).to eq user_attributes[:name]

        expect(@user.roles.pluck(:name)).to include('crc_1280_vd66vz88c_manager')
        responsibility = Sipity::WorkflowResponsibility.last
        expect(responsibility.agent.proxy_for).to eq @user
        expect(responsibility.workflow_role.role.name).to eq 'approving_group_manager'
      end
    end
    context "update" do
      before do
        user_attributes = {
          email: "example_user@hyrax"
        }
        _success, _messages, @new_user = User.create_user_from_hash(user_attributes)
      end
      after do
        @new_user.delete if @new_user.present?
        @user.delete if @user.present?
      end
      it 'does not update a user by default' do
        user_attributes = {
          email: "example_user@hyrax",
          name: "Example User"
        }
        success, messages, @user = User.create_user_from_hash(user_attributes)
        expect(success).to be false
        expect(messages).to include("Warning: User #{user_attributes[:email]} already exists. Not updating user.")
        expect(@user).to eq @new_user
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'does not update a user if update_user flag is false' do
        user_attributes = {
          email: "example_user@hyrax",
          name: "Example User",
          saml_id: "hyrax:1234",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: false)
        expect(success).to be false
        expect(messages).to include("Warning: User #{user_attributes[:email]} already exists. Not updating user.")
        expect(@user).to eq @new_user
        expect(@user.email).to eq user_attributes[:email]
        expect(@user.encrypted_password).to be_present
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'updates a user password' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "ThisIsANewPassword"
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).not_to eq @new_user.encrypted_password
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'does not update a user password if empty string' do
        user_attributes = {
          email: "example_user@hyrax",
          password: ""
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).to eq @new_user.encrypted_password
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'does not update a user password if empty string with spaces' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "  "
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).to eq @new_user.encrypted_password
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'updates a user name' do
        user_attributes = {
          email: "example_user@hyrax",
          name: "Example user"
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).to eq @new_user.encrypted_password
        expect(@user.display_name).to eq user_attributes[:name]
        expect(@user.display_name).not_to eq @new_user.display_name
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to be nil
      end
      it 'updates a user saml' do
        user_attributes = {
          email: "example_user@hyrax",
          saml_id: "hyrax:1234",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).to eq @new_user.encrypted_password
        expect(@user.display_name).to be nil
        expect(@user.provider).to eq 'saml'
        expect(@user.uid).to eq user_attributes[:saml_id]
        expect(@user.orcid).to be nil
      end
      it 'updates a user Orcid' do
        user_attributes = {
          email: "example_user@hyrax",
          orcid: "0000-1234",
        }
        success, messages, @user = User.create_user_from_hash(user_attributes, update_user: true)
        expect(success).to be true
        expect(messages).to include("Updating user #{user_attributes[:email]}")
        expect(@user.email).to eq @new_user.email
        expect(@user.encrypted_password).to be_present
        expect(@user.encrypted_password).to eq @new_user.encrypted_password
        expect(@user.display_name).to be nil
        expect(@user.provider).to be nil
        expect(@user.uid).to be nil
        expect(@user.orcid).to eq user_attributes[:orcid]
      end
    end

    context "destroy" do
      after do
        @user.delete if @user.present?
      end

      it 'Delete user with corresponding role and workflow role' do
        user_attributes = {
          email: "example_user@hyrax",
          password: "password",
          name: "Example User",
          role: "crc_1280_group_manager",
          group_id: "vd66vz88c"
        }
        _success, _messages, @user = User.create_user_from_hash(user_attributes)
        expect(@user.roles.pluck(:name)).to include('crc_1280_vd66vz88c_manager')
        # Checks before delete
        responsibility = Sipity::WorkflowResponsibility.last
        expect(responsibility.agent.proxy_for).to eq @user
        expect(responsibility.workflow_role.role.name).to eq 'approving_group_manager'
        @user.destroy
        # Check after delete
        new_responsibility = Sipity::WorkflowResponsibility.last
        expect(new_responsibility.agent.proxy_for).not_to eq @user
        expect(new_responsibility.workflow_role.role.name).not_to eq 'approving_group_manager'
      end
    end
  end
end
